﻿local ALPHA = "ALPHA"
local RELEASE = "RELEASE"
local releaseString = nil
local addon = LibStub("AceAddon-3.0"):NewAddon("EveryQuest", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
EveryQuest = addon
local L = LibStub("AceLocale-3.0"):GetLocale("EveryQuest")
local locale = GetLocale()
local releaseRevision = nil
do -- Borrowed from BigWigs :)
	-- START: MAGIC WOWACE VOODOO VERSION STUFF
	local releaseType = RELEASE

	-- This will (in ZIPs), be replaced by the highest revision number in the source tree.
	releaseRevision = tonumber("007")

	-- If the releaseRevision ends up NOT being a number, it means we're running a SVN copy.
	if type(releaseRevision) ~= "number" then
		releaseRevision = -1
	end

	-- Then build the release string, which we can add to the interface option panel.
	local majorVersion = GetAddOnMetadata("EveryQuest", "Version") or "2.?"
	if releaseRevision == -1 then
		releaseString = L["You are running a source checkout of EveryQuest %s directly from the repository."]:format(majorVersion)
	elseif releaseType == ALPHA then
		releaseString = L["You are running an ALPHA RELEASE of EveryQuest %s (revision %d)."]:format(majorVersion, releaseRevision)
	end
	_G.EVERYQUEST_RELEASE_TYPE = releaseType
	_G.EVERYQUEST_RELEASE_REVISION = releaseRevision
	_G.EVERYQUEST_RELEASE_STRING = releaseString
end

local db, dbpc
local faction, zoneid, zonegroup, clickedID, recentquest, recentcompleted, recentcategory
addon.zoneid = zoneid
addon.zonegroup = zonegroup
addon.cshown, addon.cprogress, addon.cdone, addon.cunknown = 0,0,0,0
EveryQuestData = {}
local questdisplay = {}
EQ_TextColors = {}
local format = string.format
EQ_TextColors["s0"] = { r = 1.00, g = 1.00, b = 0, font = EQFont_Progress, hex = "ffff00" };
EQ_TextColors["s2"] = { r = 0.0, g = 0.807, b = 0.019, font = EQFont_TurnedIn, hex = "00cd04" };
EQ_TextColors["unknown"]	= { r = 1, g = 1, b = 1, font = EQFont_Unknown, hex = "ffffff" };
EQ_TextColors["ignored"] = { r = 0.733, g = 0.372, b = 1, font = EQFont_Ignored, hex = "BB5FFF" };

local currentsort = {}
addon.listsorted = false

-- Short form quest types, used in quest names (Buttons)
local qtypes = {}

local new, del, deepDel
do
	addon.tableCache = setmetatable({},{__mode='k'})
	function new(...)
		local t = next(addon.tableCache)
		if t then
			addon.tableCache[t] = nil
			for i = 1, select('#', ...) do
				t[i] = select(i, ...)
			end
			-- EveryQuest:Debug("Returned recycled table")
			return t
		else
			-- EveryQuest:Debug("New table made")
			return { ... }
		end
	end
	function del(t)
		for k in pairs(t) do
			t[k] = nil
		end
		-- EveryQuest:Debug("Recycle table")
		addon.tableCache[t] = true
		return nil
	end
	
	function deepDel(t)
		if type(t) ~= "table" then
			return nil
		end
		-- EveryQuest:Debug("Deep recycle table")
		for k,v in pairs(t) do
			t[k] = deepDel(v)
		end
		return del(t)
	end
	addon.new, addon.del, addon.deepDel = new, del, deepDel
end

function addon:GetReleaseString()
	return releaseString
end

function addon:Error(string)
	self:Print("|cffFF3232", string, "|r")
end
function addon:Debug(string)
	if self.db.profile.debug then  
		self:Print(string)
	end
end
function addon:Colorize(hexColor, text)
	return "|cff" .. tostring(hexColor or 'ffffff') .. tostring(text) .. "|r"
end
function addon:RGB2Hex(r,g,b)
	-- addon:Debug(format("%02x%02x%02x",r*255,g*255,b*255))
	return format("%02x%02x%02x",r*255,g*255,b*255)
end

-- Get Player (Race/Class/Faction) for Filters - Fixed Variables
local PlayerRace = select(3, UnitRace("player")) -- Race Number
local PlayerClass = select(2, UnitClassBase("player")) -- Class Number
local PlayerFaction = select(2, UnitFactionGroup("player")) -- Faction Name
local function getRace(race)
	if PlayerRace == race then return true else return false end
end
local function getClass(class)
	if PlayerClass == class then return true else return false end
end
local function getfaction(side)
	if PlayerFaction == side then return true else return false end
end

function addon:OnInitialize()
	if PlayerFaction == "Alliance" then
		faction = 1
	else
		faction = 2
	end
	---------------------------------------------------------------------
	
	--- Setup Databases, Options
    self.db = LibStub("AceDB-3.0"):New("EQ2DB", {
			profile = {
				debug = false,
				colors = {
					Failed = {1, 0, 0},
					Progress = {1, 1, 0},
					Complete = {0, 1, 0},
					TurnedIn = {0, 0.807, 0.019},
					Unknown = {1, 1, 1},
					Ignored = {0.733, 0.372, 1},
					Repeat = {0.094, 0.627, 0.94},
				},
				localized = {
					["*"] = {},
				},
			},
		}, "global")
	db = self.db.profile
	self.dbpc = LibStub("AceDB-3.0"):New("EQ2DBPC", {
			profile = {
		        zoneid = nil,
				dbversion = 2,
				zonegroup = nil,
				history = {},
				enabledfilters = {
					Faction = true,
					Race = true,
					Class = true,
					Status = true,
				},
				enabledsort = {
					["*"] = true,
				},
				sort = {
					["t"] = { -- Default Type Sort
						["Weight"] = 1,
						["Dir"] = 1,
					},
					["l"] = { -- Default Level Sort
						["Weight"] = 2,
						["Dir"] = 1,
					},
					["n"] = { -- Default Name Sort
						["Weight"] = 0,
						["Dir"] = 1,
					},
				},
				filters = {
					-- Status
					Repeatable = true,
					TurnedIn = true,
					Ignored = false,
					Unknown = true,
					InProgress = true,
					-- Faction
					Alliance = getfaction("Alliance"),
					Horde = getfaction("Horde"),
					SideBoth = true,
					SideNone = true,
					-- Level
					PlayerLevel = UnitLevel("player"),
					-- Type
					NormalQuest = true,
					GroupQuest = true,
					DungeonQuest = true,
					RaidQuest = true,
					PvpQuest = true,
					-- Race
					Human = getRace("23"),
					Dwarf = getRace("3"),
					NightElf = getRace("4"),
					Gnome = getRace("7"),
					Orc = getRace("2"),
					Undead = getRace("5"),
					Tauren = getRace("6"),
					Troll = getRace("8"),
					-- Class
					WARRIOR = getClass(1),
					PALADIN = getClass(2),
					HUNTER = getClass(3),
					ROGUE = getClass(4),
					PRIEST = getClass(5),
					SHAMAN = getClass(7),
					MAGE = getClass(8),
					WARLOCK = getClass(9),
					DRUID = getClass(11),
				},
		    },
		}, "global")
	dbpc = self.dbpc.profile
	
	self:SetupOptions()
	
	if dbpc.dbversion < 4 then
		self:SilentUpgrade()
	end
	self:SetupFrames()
	qtypes[1] = L["G"] -- Short for 'Group'
	qtypes[41] = L["P"] -- Short for 'PvP'
	qtypes[62] = L["R"] -- Short for 'Raid'
	qtypes[81] = L["D"] -- Short for 'Dungeon'
	qtypes[84] = L["E"] -- Short for 'Escort'
	addon:UpdateSortList()
	
	self:RegisterMessage("EveryQuestDisplayChanged", "UpdateFrame")
end

function addon:ModGetDB()
	return db, dbpc
end

local continentmap = {
	[1] = "Kalimdor",
	[2] = "Eastern Kingdoms"
}

function addon:GetZoneId(category, continent)
	for k,v in pairs(EQ_zones) do -- for each top level category
		if v[1] == category then
			return nil, v[1]
		else
			if continent ~= nil then
				if v[1] == continentmap[continent] then
					for ak,av in pairs(v[2]) do -- for each category
						if av[2] == category then
							return av[1], v[1]
						end
					end
				end
			else
				for ak,av in pairs(v[2]) do -- for each category
					if type(av[2]) == "table" then
						for bk,bv in pairs(av[2]) do
							if bv[2] == category then
								return bv[1], v[1]
							end
						end
					else
						if av[2] == category then
							return av[1], v[1]
						end
					end
				end
			end
		end
	end
end

function addon:GetCategory(zid)
	for k,group in pairs(EQ_zones) do -- for each top level category
		for ak,category in pairs(group[2]) do -- for each category -- category[1] = Zone ID, [2] = Zone Name
			-- if the second item in the table is a table, then it must be a sub menu (Dungeons)
			if type(category[2]) == "table" then
				for bk,dungeon in pairs(category[2]) do -- dungeon[1] = Zone ID, [2] = Zone Name
					if dungeon[1] == zid then
						return group[1], dungeon[2]
					end
				end
			else
				if category[1] == zid then
					return group[1], category[2]
				end
			end
		end
	end
end

-- menu create function
do
	local info = {}
	local EveryQuest_ZoneMenu = CreateFrame("Frame", "EveryQuest_ZoneMenu")
	EveryQuest_ZoneMenu.displayMode = "MENU"
	EveryQuest_ZoneMenu.point = "TOPRIGHT"
	EveryQuest_ZoneMenu.relativeTo = "EQ_MenuButton"
	EveryQuest_ZoneMenu.relativePoint = "BOTTOMRIGHT"
	EveryQuest_ZoneMenu.initialize = function(self, level)
		if not level then return end
		--sort(EQ_zones, function(a, b) addon:Debug(a .. " - " .. b) return true end)
		if (level == 1) then
			-- List the Modules
			for key,value in pairs(EQ_zones) do
				wipe(info)
				info.hasArrow = true
				info.notCheckable = true
				info.text = value[3]
				info.value = key
				info.keepShownOnClick = 1
				UIDropDownMenu_AddButton(info, level)
			end
			
			-- Close menu item
			wipe(info)
			info.text         = CLOSE
			info.func         = function() CloseDropDownMenus() end
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
		end -- if level 1
		if level == 2 then
			-- Second level header (Module name)
			wipe(info)
			curzone = EQ_zones[UIDROPDOWNMENU_MENU_VALUE]
			info.isTitle      = 1
			info.text         = curzone[3]
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			
			-- List the zones
			for key, value in pairs(curzone[2]) do
				wipe(info)
				-- List the zones in this module
					info.text = value[2]
					info.value = value[1]
					info.checked = function() if zoneid == value[1] then return true end end
					info.func = function()
						zoneid = value[1]
						zonegroup = curzone[1]
						CloseDropDownMenus()
						addon.listsorted = false
						addon:SendMessage("EveryQuestDisplayChanged", value[1])
					end
				UIDropDownMenu_AddButton(info, level);
			end
			
			-- Close menu item
			wipe(info)
			info.text         = CLOSE
			info.func         = function() CloseDropDownMenus() end
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

do
	local info = {}
	local EveryQuest_QuestMenu = CreateFrame("Frame", "EveryQuest_QuestMenu")
	EveryQuest_QuestMenu.displayMode = "MENU"
	EveryQuest_QuestMenu.initialize = function(self, level)
	if not level then return end
		for k in pairs(info) do info[k] = nil end
	end
end

do
	-- To fix Blizzard's bug caused by the new "self:SetFrameLevel(2);"
	local function FixFrameLevel(level, ...)
		for i = 1, select("#", ...) do
			local button = select(i, ...)
			button:SetFrameLevel(level)
		end
	end
	function addon.FixMenuFrameLevels()
		local f = DropDownList1
		local i = 1
		while f do
			FixFrameLevel(f:GetFrameLevel() + 2, f:GetChildren())
			i = i + 1
			f = _G["DropDownList"..i]
		end
	end

	-- To fix Blizzard's bug caused by the new "self:SetFrameLevel(2);"
	hooksecurefunc("UIDropDownMenu_CreateFrames", addon.FixMenuFrameLevels)
end

function addon:OnEnable()
	addon:EventsEnable()
end

function addon:OnDisable()
	addon:UnhookAll()
end

function addon:Toggle()
	if EveryQuestFrame:IsShown() then
		EveryQuestFrame:Hide()
	else
		EveryQuestFrame:Show()
		self:UpdateFrame()
	end
end
function addon:OnShow()
	if dbpc.posx and dbpc.posy then
		EveryQuestFrame:ClearAllPoints()
		EveryQuestFrame:SetPoint("TOPLEFT","UIParent", "BOTTOMLEFT", dbpc.posx, dbpc.posy)
	end
end

function addon:SavePosition()
	local Left = EveryQuestFrame:GetLeft()
	local Top = EveryQuestFrame:GetTop()
	if Left and Top then
		dbpc.posx = Left
		dbpc.posy = Top
	end
end

function addon:ResetPosition()
	dbpc.posx = nil
	dbpc.posy = nil
end

function addon:SilentUpgrade()
	local t = {}
	for zone, quests in pairs(dbpc.history) do
		if quests["status"] == nil then
			for uid, quest in pairs(quests) do
				if quest["status"] ~= nil then
					t[uid] = quest
				end
			end
		end
	end
	dbpc.history = t
	
	for uid, quest in pairs(dbpc.history) do
		dbpc.history[uid].id = nil
	end
	dbpc.dbversion = 4
end

function addon:AddUnknown(uid, category, title)
	if uid and uid ~= nil then
		if db.unknowns == nil then
			db.unknowns = {}
		end
		if db.unknowns[uid] == nil then
			db.unknowns[uid] = EVERYQUEST_RELEASE_REVISION .. "||" .. date("%x") .. "||" .. tostring(title) .. "||" .. tostring(category)
		end
	end
end

local classes = {[1]="WARRIOR",[2]="PALADIN",[3]="HUNTER",[4]="ROGUE",[5]="PRIEST",[7]="SHAMAN",[8]="MAGE",[9]="WARLOCK",[11]="DRUID"}
local races = {[23]="Human",[2]="Orc",[3]="Dwarf",[4]="NightElf",[5]="Undead",[6]="Tauren",[7]="Gnome",[8]="Troll"}

do
	local filters
	local enabledfilters
	local side = false
	local status = false
	local level = false
	local qtype = false
	local ignored = true
	local level = false
	local race = false
	local class = false
	function addon:Filterer(uid, quest, overrideStatus)
		filters = dbpc.filters
		enabledfilters = dbpc.enabledfilters
		side = false
		status = false
		level = false
		qtype = false
		ignored = true
		level = false
		race = false
		class = false
		-- Faction Filter
		if enabledfilters.Faction then
			if quest.s == 1 and filters.Alliance then side = true end
			if quest.s == 2 and filters.Horde then side = true end
			if quest.s == 3 and filters.SideBoth then side = true end
			if (quest.s == 0 or quest.s == nil) and filters.SideNone then side = true end
		else
			side = true
		end
		-- Race Filter
		if enabledfilters.Race and quest.rc ~= nil then			
			for count in pairs(quest.rc) do -- For all Classes listed
				races = quest.rc[count]
				if races == PlayerClass then
					race = true
					break
				end
			end
		else
			race = true
		end
		-- Class Filter
		if enabledfilters.Class and quest.cl ~= nil then
			for count in pairs(quest.cl) do -- For all Classes listed
				classes = quest.cl[count]
				if classes == PlayerClass then
					class = true
					break
				end
			end
		else
			class = true
		end
		-- Type Filter
		if enabledfilters.Type then
			if quest.t == 1 and filters.GroupQuest then
				qtype = true
			elseif quest.t == 41 and filters.PvpQuest then
				qtype = true
			elseif quest.t == 62 and filters.RaidQuest then
				qtype = true
			elseif quest.t == 81 and filters.DungeonQuest then
				qtype = true
			elseif quest.t == nil and filters.NormalQuest then
				qtype = true
			end
		else
			qtype = true
		end
		-- Status Filter
		if enabledfilters.Status then
			local qstatus = nil
			if dbpc.history[uid] then
				qstatus = dbpc.history[uid].status
			end
			if qstatus == 0 and filters.InProgress then
				status = true
			elseif qstatus == 2 and filters.TurnedIn then
				status = true
			elseif quest.rr == 1 and filters.Repeatable then
				status = true
			elseif quest.i == 1 and filters.Ignored then
				status = true
			elseif qstatus == nil and filters.Unknown and quest.rr ~= 1 and quest.i ~= 1 then
				status = true
			end
		else
			status = true
		end
		-- Level Filter
		if enabledfilters.Level then
			local qlevel
			if quest.l == nil and quest.r == nil then
				qlevel = 0
			elseif quest.r == nil then
				qlevel = quest.l
			else
				qlevel = quest.r
			end
			if qlevel <= filters.PlayerLevel then
				level = true
			end
		else
			level = true
		end
		
		if side and status and qtype and level and race and class then
			return true
		else 
			return false
		end
	end
end -- end filterer

function addon:UpdateFrame(e, zid)
	local zonename
	if zid ~= nil then
		zoneid, zonegroup, zonename = zid, addon:GetCategory(zid)
	else
		_, zonename = self:GetCategory(zoneid)
	end
	addon.cprogress, addon.cdone, addon.cunknown = 0,0,0,0
	dbpc.zoneid = zoneid
	dbpc.zonegroup = zonegroup
	-- 
	UIDropDownMenu_SetText(EQDropdown, zonename)
	if EveryQuestFrame:IsShown() then
		--[[if db.locallist then
			self:Debug("Localized quest list is on")
		end]]
		local buttonid = 1
		local controli = 0
		questlist = self:GetZoneQuestData(zonegroup, zoneid)
		-- Spew("questlist", questlist)
		local questcount = 0
		if questlist then
			for k, quest in pairs (questlist) do
				if addon:Filterer(quest.id, quest) then
					if dbpc.history[quest.id] ~= nil then
						if dbpc.history[quest.id].status == 0 then
							addon.cprogress = addon.cprogress + 1
						elseif dbpc.history[quest.id].status == 2 then
							addon.cdone = addon.cdone + 1
						end
					else
						addon.cunknown = addon.cunknown +1
					end
					questcount = questcount +1
				end
			end
		end
		cshown = questcount
		
		if questcount > 0 and not addon.listsorted then
			--self:Print("Sort the list")
			sort(questlist, function(a,b) return self:SortTable(a,b) end)
			addon.listsorted = true
		end
		FauxScrollFrame_Update(EveryQuestListScrollFrame,questcount,27,16)
		local scrolloffset = FauxScrollFrame_GetOffset(EveryQuestListScrollFrame)
		local quest
		if questcount then
			if questlist then
				for k, quest in pairs (questlist) do
					-- if quest["s"] then
						if addon:Filterer(quest.id, quest) then
							controli = controli + 1
							if controli > scrolloffset then
								if buttonid > 27 then 
									break 
								end
								if dbpc.history[quest.id] ~= nil then
									quest.status = dbpc.history[quest.id].status
								end
								
								self:UpdateButton(buttonid, quest, quest.id)
								buttonid = buttonid +1		
							end
						end
					-- end
				end
			end
		end
		for j = buttonid, 27 ,1 do
			button = getglobal("EveryQuestTitle"..j)
			button:Hide()
		end
		addon.frames.Shown:SetText( format(L["%d Shown"], questcount) )  -- Main frame list count
	end
end
--- Get Quest History data for the specified Quest ID
function addon:GetHistory(uid)
	if dbpc.history[uid] then
		return uid, dbpc.history[uid]
	end
end
--- Get Quest status for the specified Quest ID
function addon:GetHistoryStatus(uid)
	if dbpc.history[uid] then
		return dbpc.history[uid].status
	else
		return -2
	end
end

local function mergequest(quest1, quest2)
	local quest1, quest2 = quest1, quest2
	if quest2 then
		for k,v in ipairs(quest2) do table.insert(quest1, v) end return quest1
	end
	return quest1
end

function addon:BuildButtonText(uid, quest)
	local qTag = ""
	local strLM = ""
	local level
	local text
	
	if quest.t then -- quest type
		qTag = qtypes[quest.t]
	end
	if quest.l then -- quest level
		level = quest.l
	else 
		if quest.r then
			level = format(L["r%d"], quest.r) -- Short for 'required level'
		else
			level = "--"
		end
	end
	button_text = format("[%s%s] %s%s", level, qTag, quest.n, strLM) -- Generates Quest Name
	local color
	if dbpc.history[uid] then
		local his = dbpc.history[uid]
		if his.status ~= nil then
			if his.status == 0 then
				color = addon:RGB2Hex(unpack(db.colors.Progress))
			elseif his.status == 1 then
				color = addon:RGB2Hex(unpack(db.colors.Complete))
			elseif his.status == 2 then
				color = addon:RGB2Hex(unpack(db.colors.TurnedIn))
			end
		else
			if not repeating then
				color = addon:RGB2Hex(unpack(db.colors.Unknown))
			end
		end
	else
		if not repeating then
			color = addon:RGB2Hex(unpack(db.colors.Unknown))
		end
	end
	
	if quest.rr then
		color = addon:RGB2Hex(unpack(db.colors.Repeat))
	end
	if quest.i then
		color = addon:RGB2Hex(unpack(db.colors.Ignored))
	end
	
	return addon:Colorize(color, button_text)
end

function addon:UpdateButtonTextByUID(uid)
	for k,quest in pairs(questdisplay) do
		if quest.id == uid then
			button:SetText(addon:BuildButtonText(uid, quest))
			break
		end
	end
end

function addon:UpdateButton(buttonid, quest, uid)
	local button = getglobal("EveryQuestTitle"..buttonid)
	button:SetNormalTexture("")
	button:SetText("")
	if not questdisplay[buttonid] or questdisplay[buttonid].id ~= uid then
		questdisplay[buttonid] = nil
		questdisplay[buttonid] = mergequest(quest, dbpc.history[uid])
		questdisplay[buttonid].id = uid
	end
	
	button:SetText(addon:BuildButtonText(uid, quest))
	button:Show()
end

function addon:ButtonEnter(self)
	local index = self:GetID()
	local isCollected = false
	local quest = questdisplay[index]
	local zoneid = zoneid
	local uid = quest.id
end

function addon:GetStatus(displayid, status, ignored)
	--self:Debug("GetStatus - did:"..tostring(displayid)..", status:"..tostring(queststatus))
	if questdisplay[displayid].status and questdisplay[displayid].status == status then
		return true
	else
		return false
	end
end

local function pairsByKeys (t, f)
      local a = {}
      for n in pairs(t) do table.insert(a, n) end
      table.sort(a, f)
      local i = 0      -- iterator variable
      local iter = function ()   -- iterator function
        i = i + 1
        if a[i] == nil then return nil
        else return a[i], t[a[i]]
        end
      end
      return iter
    end

function addon:UpdateSortList()
	--wipe(currentsort)
	--self:Print("UpdateSortList")
	local list = {}
	local list2 = {}
	for k,v in pairs(dbpc.sort) do
		if dbpc.enabledsort[k] then
			if list[v["Weight"]] == nil then
				list[v["Weight"]] = k
			else
				list[v["Weight"] +1] = k
			end
			--self:Print(k)
		end
	end
	for k,v in pairsByKeys(list) do
		list2[#list2 + 1] = v
		--self:Print(v)
	end
	--Spew("list", list)
	currentsort = list2
	addon.listsorted = false
	--Spew("list2", list2)
end

function addon:SortTable(aa,bee)
	local f = currentsort
	local a = new()
	local b = new()
	a.l = (aa.l ~= nil and aa.l) or (aa.r ~= nil and aa.r) or 0
	b.l = (bee.l ~= nil and bee.l) or (bee.r ~= nil and bee.r) or 0
	a.d = aa.d
	b.d = bee.d
	a.w = aa.w
	b.w = bee.w
	a.n = db.localized[locale][aa.id] ~= nil and db.localized[locale][aa.id] or aa.n
	b.n = db.localized[locale][bee.id] ~= nil and db.localized[locale][bee.id] or bee.n
	a.t = aa.t
	b.t = bee.t
	if a["l"] == nil then if a["r"] == nil then a["l"] = 0 else a["l"] = a["r"] end end
	if b["l"] == nil then if b["r"] == nil then b["l"] = 0 else b["l"] = b["r"] end end
	if (a[f[5]] or 0) == (b[f[5]] or 0) then
		if (a[f[4]] or 0) == (b[f[4]] or 0) then
			if (a[f[3]] or 0) == (b[f[3]] or 0) then
				if (a[f[2]] or 0) == (b[f[2]] or 0) then
					if (a[f[1]] or 0) == (b[f[1]] or 0) then
						return false
					else
						if dbpc.sort[f[1]].Dir == 2 then
							return (a[f[1]] or 0) > (b[f[1]] or 0)
						else
							return (a[f[1]] or 0) < (b[f[1]] or 0)
						end
					end
				else
					if dbpc.sort[f[2]].Dir == 2 then
						return (a[f[2]] or 0) > (b[f[2]] or 0)
					else
						return (a[f[2]] or 0) < (b[f[2]] or 0)
					end
				end
			else
				if dbpc.sort[f[3]].Dir == 2 then
					return (a[f[3]] or 0) > (b[f[3]] or 0)
				else
					return (a[f[3]] or 0) < (b[f[3]] or 0)
				end
			end
		else 
			if dbpc.sort[f[4]].Dir == 2 then
				return (a[f[4]] or 0) > (b[f[4]] or 0)
			else
				return (a[f[4]] or 0) < (b[f[4]] or 0)
			end
		end
	else 
		if dbpc.sort[f[5]].Dir == 2 then
			return (a[f[5]] or 0) > (b[f[5]] or 0)
		else
			return (a[f[5]] or 0) < (b[f[5]] or 0)
		end
	end
	del(a)
	del(b)
end

-- GetZoneQuestData
function addon:GetZoneQuestData(zonegroup, zoneid)
	groupdata = self:LoadQuestData(zonegroup)
	if groupdata ~= false then
		return groupdata[zoneid]
	else
		return false
	end
end
-- LoadQuestData
function addon:LoadQuestData(group)
	if group ~= nil then
		return EveryQuestData[group]
	else
		return false
	end
end

function addon:GetColor(hex)
	if hex == 2 then
		return addon:RGB2Hex(unpack(db.colors.TurnedIn))
	elseif hex == 0 then
		return addon:RGB2Hex(unpack(db.colors.Progress))
	elseif hex == -2 then -- Unknown
		return addon:RGB2Hex(unpack(db.colors.Unknown))
	end
	local rhex, ghex, bhex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6)
	return tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255
end

function EveryQuest_ScrollFrame_Update()
	addon:UpdateFrame()
end

--- EOF ---