local addon = LibStub("AceAddon-3.0"):GetAddon("EveryQuest")
local L = LibStub("AceLocale-3.0"):GetLocale("EveryQuest")
local zoneid, zonegroup
local db, dbpc

function addon:EventsEnable()
	self:RegisterEvent("PLAYER_LEVEL_CHANGED") -- Used to update level filter
	self:RegisterEvent("PLAYER_ENTERING_WORLD") -- Used in query.lua
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterMessage("EveryQuestDisplayChanged", "UpdateFrame")
	-- Quest Related Events
	self:RegisterEvent("QUEST_ACCEPTED") -- When player accepts quests
	self:RegisterEvent("QUEST_ACCEPT_CONFIRM") -- When player accepts escort quest started by another player
	self:RegisterEvent("QUEST_TURNED_IN") -- When player Turns in a quest
	self:RegisterEvent("QUEST_REMOVED") -- When Player Abandons quest/Also triggered when player turns in a quest
	
	db = EveryQuest.db.profile
	dbpc = EveryQuest.dbpc.profile
end

function addon:PLAYER_LEVEL_CHANGED(event, oldLevel, newLevel)
	dbpc.filters.PlayerLevel = newLevel
	addon:SendMessage("EveryQuestDisplayChanged")
end

function addon:ZONE_CHANGED_NEW_AREA()
	z, g = self:GetZoneId(GetRealZoneText())
	if z ~= nil then
		zoneid = z
	end
	if g ~= nil then
		zonegroup = g
	end
	self:Debug("Zone Changed ("..GetRealZoneText()..") - zoneid = " .. tostring(zoneid) .. " zonegroup = " .. tostring(zonegroup))
	addon.listsorted = false
	addon:SendMessage("EveryQuestDisplayChanged", z)
end

function addon:CheckDB(questid)
	if dbpc.history[questid] == nil then -- If the quest does not exist in the DB
		dbpc.history[questid] = {}		 -- Create the Quest Entry
	end
end

function addon:QUEST_ACCEPTED(event, QuestLogindex, questid)
	self:Debug("QUEST_ACCEPTED - "..tostring(questid))
	addon:CheckDB(questid)
	dbpc.history[questid].status = 0 -- In Progress		
	if Relationships[questid] then -- Check to see if BreadCrumb exists
		for count in pairs(Relationships[questid].bc) do -- For all BreadCrumbs listed
			breadc = Relationships[questid].bc[count]
			if dbpc.history[breadc] == nil or dbpc.history[breadc].status ~= 2 then -- If it doesn't exist or not turned in
				EveryQuest:Print(string.format(L["This Quest has a Breadcrumb you could complete = ID: %d"], breadc))
			end
		end
	end
	addon:SendMessage("EveryQuestDisplayChanged")
end

function addon:QUEST_ACCEPT_CONFIRM(event, QuestLogindex, questid)
	self:Debug("QUEST_ACCEPTED - "..tostring(questid))
	addon:CheckDB(questid)
	dbpc.history[questid].status = 0 -- In Progress
	addon:SendMessage("EveryQuestDisplayChanged")
end

function addon:QUEST_REMOVED(event, questid)
	addon:CheckDB(questid)
	if dbpc.history[questid].status ~= 2 then -- Quest is removed on abandon and after turnedIn
		self:Debug("QUEST_REMOVED - "..tostring(questid))
		dbpc.history[questid] = nil -- Abandon
		addon:SendMessage("EveryQuestDisplayChanged")
	end
end

function addon:QUEST_TURNED_IN(event, questid)
	self:Debug("QUEST_COMPLETE - "..tostring(questid))
	addon:CheckDB(questid)
	dbpc.history[questid].status = 2 -- TurnedIn (Completed)
	addon:SendMessage("EveryQuestDisplayChanged")
end