﻿local EveryQuest = LibStub("AceAddon-3.0"):GetAddon("EveryQuest")
local addon = EveryQuest
local L = LibStub("AceLocale-3.0"):GetLocale("EveryQuest")
local db, dbpc
local options, moduleOptions = nil, {}
local format = string.format
local sortcommon = {
	Weight = {
		type = 'range',
		name = L["Weight"],
		min = -10,
		max = 10,
		step = 1,
		order = 3,
	},
	Dir = {
		type = 'select',
		name = L["Direction"],
		style = "dropdown",
		values = { -- A hack to sort them in the menu
			L["Ascending"],
			L["Descending"],
		},
		order = 4,
	},
}

local generalOptions = {
	order = 1,
	type = "group",
	name = L["EveryQuest"],
	get = function(info) return db[ info[#info] ] end,
	set = function(info, value) db[ info[#info] ] = value end,
	args = {
		intro = {
			order = 1,
			type = "description",
			name = L["EQ_DESC"],
		},
		Colors = {
			get = function(info) return unpack(db.colors[ info[#info] ]) end,
			set = function(info, r, g, b, a) db.colors[ info[#info] ] = {r, g, b, a} EveryQuest:SendMessage("EveryQuestDisplayChanged") end,
			type = "group",
			order = 2,
			guiInline = true,
			name = L["Quest Colors"],
			args = {
				Unknown = {
					order = 5,
					type = 'color',
					name = L["Unknown Status"],
				},
				Progress = {
					order = 4,
					type = 'color',
					name = L["In Progress"],
				},
				TurnedIn = {
					order = 2,
					type = 'color',
					name = L["Turned In"],
				},
				Ignored = {
					order = 7,
					type = 'color',
					name = IGNORED,
				},
				Repeat = {
					order = 1,
					type = 'color',
					name = L["Repeatable"],
				}
			},
		},
		header = {
			order = 6,
			type = 'header',
			name = CALENDAR_TYPE_OTHER,
		},
		debug = {
			order = 7,
			type = 'toggle',
			name = L["Show Debugging Messages"],
			desc = L["Show Debugging Messages - *WARNING* Spams your default chat frame"],
			width = "double",
		},
	},
}
local cc = RAID_CLASS_COLORS
local filterOptions = {
	type = "group",
	name = L["Filters"],
	get = function(info) return dbpc.filters[ info[#info] ] end,
	set = function(info, value) dbpc.filters[ info[#info] ] = value EveryQuest:SendMessage("EveryQuestDisplayChanged") end,
	args = {
		intro = {
			order = 1,
			type = "description",
			name = L["Configure filtering options for the quest list. These settings are character specific."],
		},
		UseableFilters = {
			get = function(info) return dbpc.enabledfilters[ info[#info] ] end,
			set = function(info, value) dbpc.enabledfilters[ info[#info] ] = value EveryQuest:SendMessage("EveryQuestDisplayChanged") end,
			type = "group",
			order = 2,
			guiInline = true,
			name = L["Enable Filtering"],
			args = {
				Faction = {
					type = 'toggle',
					name = L["Faction Filter"],
				},
				Level = {
					type = 'toggle',
					name = L["Level Filter"],
				},
				Type = {
					type = 'toggle',
					name = L["Type Filter"],
				},
				Status = {
					type = 'toggle',
					name = L["Status Filter"],
				},
				Race = {
					type = 'toggle',
					name = L["Race Filter"],
				},
				Class = {
					type = 'toggle',
					name = L["Class Filter"],
				}
			},
		},
		Status = {
			type = "group",
			order = 5,
			guiInline = true,
			disabled = function() return not dbpc.enabledfilters.Status end,
			name = L["Control the display of specific quest statuses"],
			desc = L["Control the display of specific quest statuses"],
			args = {
				Repeatable = {
					order = 1,
					type = 'toggle',
					name = function() return addon:Colorize(addon:RGB2Hex(unpack(db.colors.Repeat)), L["Repeatable"]) end,
					desc = L["Shows Repeatable quests"],
					width = "double",
				},
				TurnedIn = {
					order = 2,
					type = 'toggle',
					name = function() return addon:Colorize(addon:RGB2Hex(unpack(db.colors.TurnedIn)), L["Turned In Quests"]) end,
					desc = L["Shows quests turned back into NPCs"],
					width = "double",
				},
				InProgress = {
					order = 4,
					type = 'toggle',
					name = function() return addon:Colorize(addon:RGB2Hex(unpack(db.colors.Progress)), L["Quests In Progress"]) end,
					desc = L["Shows quests that you are on but not completed"],
					width = "double",
				},
				Unknown = {
					order = 6,
					type = 'toggle',
					name = function() return addon:Colorize(addon:RGB2Hex(unpack(db.colors.Unknown)), L["Unknown Status"]) end,
					desc = L["Shows quests you haven't seen before"],
					width = "double",
				},
				Ignored = {
					order = 7,
					type = 'toggle',
					name = function() return addon:Colorize(addon:RGB2Hex(unpack(db.colors.Ignored)), L["Ignored Quests"]) end,
					desc = L["Shows quests you've ignored"],
					width = "double",
				},
				example = {
					order = 8,
					type = "description",
					name = L["Example for Ignored quest: Some quests are still in Wowhead's database but are not attainable ingame anymore. Ignore them to hide them from the list."],
					width = "double",
				},
			},
		},
		Type = {
			type = "group",
			order = 7,
			guiInline = true,
			disabled = function() return not dbpc.enabledfilters.Type end,
			name = L["Filter Quests by Type"],
			args = {
				NormalQuest = {
					order = 1,
					type = 'toggle',
					name = L["Normal Quests"],
					desc = L["Shows normal Blizzard soloable quests"],
				},
				GroupQuest = {
					order = 2,
					type = 'toggle',
					name = L["Group Quests"],
					desc = L["Shows 'group suggested' quests"],
				},
				DungeonQuest = {
					order = 5,
					type = 'toggle',
					name = L["Dungeon Quests"],
					desc = L["Shows dungeon quests"],
				},
				RaidQuest = {
					order = 6,
					type = 'toggle',
					name = L["Raid Quests"],
					desc = L["Shows 'raid suggested' quests"],
				},
				PvpQuest = {
					order = 7,
					type = 'toggle',
					name = L["PvP Quests"],
					desc = L["Shows PVP quests"],
				},
			},
		},
	},
}

local sortOptions = {
	order = 2,
	type = "group",
	name = L["List Order"],
	args = {
		intro = {
			order = 1,
			type = "description",
			name = L["Customize how your quest list is sorted.  Higher Weight moves the items to the top of the list, lower weight goes to the bottom.  Sort order Ascending: A-Z 0-9, Descending: Z-A, 9-0."],
		},
		t = {
			type = "group",
			order = 5,
			guiInline = true,
			name = L["Order by Type"],
			get = function(info) return dbpc.sort["t"][ info[#info] ] end,
			set = function(info, value) dbpc.sort["t"][ info[#info] ] = value EveryQuest:UpdateSortList() EveryQuest:SendMessage("EveryQuestDisplayChanged") end,
			args = sortcommon,
		},
		l = {
			type = "group",
			order = 6,
			guiInline = true,
			name = L["Order by Level"],
			get = function(info) return dbpc.sort["l"][ info[#info] ] end,
			set = function(info, value) dbpc.sort["l"][ info[#info] ] = value EveryQuest:UpdateSortList() EveryQuest:SendMessage("EveryQuestDisplayChanged") end,
			args = sortcommon,
		},
		n = {
			type = "group",
			order = 7,
			guiInline = true,
			name = L["Order by Name"],
			get = function(info) return dbpc.sort["n"][ info[#info] ] end,
			set = function(info, value) dbpc.sort["n"][ info[#info] ] = value EveryQuest:UpdateSortList() EveryQuest:SendMessage("EveryQuestDisplayChanged") end,
			args = sortcommon,
		},
	},
}

local acr = LibStub("AceConfigRegistry-3.0")
local acd = LibStub("AceConfigDialog-3.0")

local function findPanel(name, parent)
	for i, button in next, InterfaceOptionsFrameAddOns.buttons do
		if button.element then
			if name and button.element.name == name then return button
			elseif parent and button.element.parent == parent then return button
			end
		end
	end
end

function addon:SetupOptions()	
	-- setup options table Mapster
	acr:RegisterOptionsTable("EveryQuest", generalOptions)
	local options = acd:AddToBlizOptions("EveryQuest", L["EveryQuest"])
	options:HookScript("OnShow", function()
		local p = findPanel("EveryQuest")
		if p and p.element.collapsed then OptionsListButtonToggle_OnClick(p.toggle) end
	end)
	
	function addon:ShowConfig()
		InterfaceOptionsFrame_OpenToCategory(options)
	end
	
	acr:RegisterOptionsTable("EveryQuest/Filters", filterOptions)
	local filters = acd:AddToBlizOptions("EveryQuest/Filters", L["Filters"], "EveryQuest")
	
	function addon:ShowFilters()
		InterfaceOptionsFrame_OpenToCategory(filters)
	end
	
	acr:RegisterOptionsTable("EveryQuest/Sort", sortOptions)
	acd:AddToBlizOptions("EveryQuest/Sort", L["List Order"], "EveryQuest")
	
	LibStub("AceConsole-3.0"):RegisterChatCommand( "everyquest", function() addon:Toggle() end)
	
	db = EveryQuest.db.profile
	dbpc = EveryQuest.dbpc.profile
end

function addon:RegisterModuleOptions(moduleName, optionsTable)
	acr:RegisterOptionsTable("EveryQuest/"..moduleName, optionsTable)
	acd:AddToBlizOptions("EveryQuest/"..moduleName, moduleName, "EveryQuest")
end

--- EOF ---