﻿local EveryQuest = LibStub("AceAddon-3.0"):GetAddon("EveryQuest")
local L = LibStub("AceLocale-3.0"):GetLocale("EveryQuest")
local MODNAME = "Query"
local Query = EveryQuest:NewModule(MODNAME, "AceEvent-3.0")
local db, dbpc
local clrimportchk = false
local addcount = 0
local completedquests = {}

local queryOptions = {
	type = "group",
	name = L["Clear Quest Data"],
	args = {
		cleardatacheck = {
			order = 4,
			type = "toggle",
			name = L["Clear Quest Data"], -- Query Options
			get = function(info) return clrimportchk end,
			set = function(info, value) clrimportchk = value end,
		},
		cleardata = {
			order = 5,
			type = "execute",
			name = L["Clear Quest Data"], -- Query Options
			desc = L["Clear Quest Data"], -- Query Options
			func = function() Query:ClearPopulated() end,
			disabled = function () if clrimportchk == true then return false else return true end end,
		},
		cleardatadesc = {
			order = 10,
			type = "description",
			name = L["CLEARDATADESC"] .. "\n", -- Query Options
		},
	}
}

function Query:OnInitialize()
	db = EveryQuest.db.profile
	dbpc = EveryQuest.dbpc.profile.history
	EveryQuest:RegisterModuleOptions(L["Clear Quest Data"], queryOptions)
end

function EveryQuest:PLAYER_ENTERING_WORLD()
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	Query:Populate()
end

function Query:Relations(questid) -- Relationships Exist between Quests
	if Relationships[questid].bc ~= nil then -- Does the Quest have a Breadcrumb?
		for count in pairs(Relationships[questid].bc) do -- For all BreadCrumbs listed
			breadc = Relationships[questid].bc[count]
			EveryQuest.dbpc.profile.history[breadc] = {}
			EveryQuest.dbpc.profile.history[breadc].status = 2
		end
	elseif Relationships[questid].ds ~= nil then -- Or does this Quest disable other quests?
		for count in pairs(Relationships[questid].ds) do -- For all BreadCrumbs listed
			disabled = Relationships[questid].ds[count]
			EveryQuest.dbpc.profile.history[disabled] = {}
			EveryQuest.dbpc.profile.history[disabled].status = 2
		end
	end	
end

function Query:Populate()
	-- Add QuestHistory
	addcount = 0
	EveryQuest:Print(L["Populating your quest history from the server, this may take a while and might freeze your game."])
	GetQuestsCompleted(completedquests)
	for questid in pairs(completedquests) do
		if Relationships[questid] then
			Query:Relations(questid)
		end
		if EveryQuest.dbpc.profile.history[questid] == nil then -- If doesn't exist
			EveryQuest.dbpc.profile.history[questid] = {}		  -- Then create it
			addcount = addcount + 1
		end
		EveryQuest.dbpc.profile.history[questid].status = 2 -- Turned in
	end
	EveryQuest:Print(string.format(L["Added %d quests to your database"], addcount))
	
	-- Add QuestLog
	numEntries, numQuests = GetNumQuestLogEntries()
	if numQuests > 0 then
		quest_count = 2
		while quest_count <= numQuests+1 do
			questid = select(8,GetQuestLogTitle(quest_count))
			if EveryQuest.dbpc.profile.history[questid] == nil then -- If doesn't exist
				EveryQuest.dbpc.profile.history[questid] = {}		  -- Then create it
			end
			EveryQuest.dbpc.profile.history[questid].status = 0
			quest_count = quest_count + 1
		end
	end
	EveryQuest:SendMessage("EveryQuestDisplayChanged")
end

function Query:ClearPopulated()
	local count = 0
	for questid, quest in pairs(dbpc) do
		dbpc[questid] = nil
		count = count + 1
	end
	EveryQuest:SendMessage("EveryQuestDisplayChanged")
	EveryQuest:Print(string.format(L["Deleted %d quests from your database"], count))
	clrimportchk = false
end

	
--- EOF ---